
let nuage = [[0;0];[1;4];[1;8];[4;1];
             [4;4];[5;9];[5;6];[7;-1];
             [7;2];[8;5];[11;6];[13;1]];;


let knuth_shuffle a =
  let n = Array.length a in
  let a = Array.copy a in
  for i = n - 1 downto 1 do
    let k = Random.int (i+1) in
    let x = a.(k) in
    a.(k) <- a.(i);
    a.(i) <- x
  done;
  a

let nuage = Array.of_list nuage in
    let ksn = knuth_shuffle nuage in
    Array.to_list ksn;;

(*[[4; 4]; [5; 6]; [5; 9]; [1; 4]; [11; 6]; [7; 2]; [4; 1]; [7; -1]; [8; 5];
 [1; 8]; [13; 1]; [0; 0]]*)

(*FF implanter un tri fusion*)





List.sort compare nuage;;

let depack l = match l with
  | [x;y]->x,y
  | _ -> failwith "pas une liste de 2 valeurs";;

depack [1;2];;
depack [1;2;3];;


(*
Renvoie 0 si pop1,p1p2 sont 2 vecteurs liés
Renvoie 1 si c'est une base orientée dans le sens direct
Renvoie -1 si c'est une base orientée dans le sens indirect
*)
let oriente p0 p1 p2 =
  let x0,y0=depack p0 in
  let x1,y1=depack p1 in
  let x2,y2=depack p2 in
  let x01,y01=x1-x0,y1-y0 in
  let x02,y02=x2-x0,y2-y0 in
  let d = x01*y02-y01*x02 in
  match d with
  | 0 -> 0
  | alpha when alpha > 0 -> 1
  | _ -> -1;;



(*es env conv supérieure;
  p : nouvelle extrémité de es
 *)
let rec majES es p =
  match es with
  | [] | [_] -> p::es
  | p1::p2::q ->
     if oriente p p1 p2 >= 0
     then p::es
     else majES (p2::q) p;;

(*
Dans la règle 3 : autant de calcul d'orientations
que d'empilement/dépilement

Idem pour les comparaisons à zéro.

Au bilan : peut-être une opération 
d'empilement/dépilement de plus que de comparaisons.

Disons aussi qu'il y a 3 fois plus de comparaisons de filtrage que
d'empilement/dépilement (puisque 3 règle de filtrage)
*)

majES
  (List.rev [[0;0]; [1;8]; [5;9]; [7;2]])
  [8;5];;

majES  [[7; 2]; [5; 9]; [1; 8]; [0; 0]] [8;5];;

(*es env conv supérieure;
  p : nouvelle extrémité de es
 *)
let rec majEI ei p =
  match ei with
  | [] | [_] -> p::ei
  | p1::p2::q ->
     if oriente p p1 p2 <= 0
     then p::ei
     else majEI (p2::q) p;;

majEI [[3;2];[4;0]] [6;1];;

let deuxEnv nuage = match nuage with
    [] | [_] | [_;_] ->
          failwith "pas assez de pts"
    | p0::p1::q ->
       let rec aux es ei c =
         match c with
         | [] -> ei, es
         | p::q ->
            aux (majES es p) (majEI ei p)
              q
       in aux [p0] [p0] (p1::q);;


(*
Avec |nuage| = n.

Chaque point est dépilé du nuage exactement une fois dans aux
avec deux fois plus de comparaisons de filtrage (2 règles).

Sur la totalité des appels à majeES, chaque point est empilé
exactement une fois et dépilé au plus 1 fois.  Donc 2n opérations
auxquelles on ajoute 2n comparaisons à zéro, 2n orientations, 6n
comparaisons de filtrage.

Idem pour majEI.

Au total :

   n     (dépilements dans aux)
+ 2n     (filtrage dans aux)
+ 2n     (empil/depil dans  majES)
+ 4n     (orientation/comparaison >=0 dans majES)
+ 6n     (comparaisons de filtrage dans majES)
+ 2n     (empil/depil dans  majEI)
+ 4n     (orientation/comparaison à zéro dans majEI)
+ 6n     (comparaisons de filtrage dans majEI)
-------------
 28n

COMPLEXITé LINEAIRE en la taille du nuage
*)


deuxEnv nuage;;
deuxEnv [[1;2];[3;4]];;


let convGraham nuage =
  let ei, es = deuxEnv nuage in
  (List.rev ei)
  @ (List.tl es);;
    
convGraham nuage;;


